@@ -2,6 +2,7 @@
 --
 -- An ECS is a modern approach to organizing your application state as a database,
 -- providing patterns for data-oriented design and parallel processing.
-module Aztecs (module Aztecs.ECS) where
+module Aztecs (module Aztecs.ECS, module Aztecs.Hierarchy) where
 
 import Aztecs.ECS
+import Aztecs.Hierarchy
